@@ -72,7 +72,7 @@ def initLocale():
         locale.setlocale( locale.LC_ALL, '' )
 
     else:
-        language_code, encoding = locale.getdefaultlocale()
+        language_code, encoding = locale.getlocale()
         if language_code is None:
             language_code = 'en_GB'
 
@@ -381,7 +381,7 @@ class SvnCommand:
     def cmd_add( self, args ):
         recurse = args.getBooleanOption( '--non-recursive', False )
         force = args.getBooleanOption( '--force', False )
-        
+
         self.client.add( args.getPositionalArgs( 1 ), recurse=recurse, force=force )
 
     def cmd_add_to_changelist( self, args ):
@@ -502,6 +502,7 @@ class SvnCommand:
     def cmd_diff( self, args ):
         recurse = args.getBooleanOption( '--non-recursive', False )
         revision1, revision2 = args.getOptionalRevisionPair( '--revision', 'base', 'working' )
+        return_bytes = args.getBooleanOption( '--return-bytes', True )
         positional_args = args.getPositionalArgs( 0, 1 )
         if len(positional_args) == 0:
             positional_args.append( '.' )
@@ -523,8 +524,13 @@ class SvnCommand:
         self.debug( 'cmd_diff %r, %r, %r, %r, %r' % (tmpdir, positional_args[0], recurse, revision1, revision2) )
         diff_text = self.client.diff( tmpdir, positional_args[0], recurse=recurse,
                                             revision1=revision1, revision2=revision2,
-                                            diff_options=['-u'] )
-        print( diff_text.replace( '\r\n', '\n' ) )
+                                            diff_options=['-u'],
+                                            return_bytes=return_bytes )
+        if return_bytes:
+            for line in diff_text.split( b'\n' ):
+                print(repr(line))
+        else:
+            print( diff_text.replace( '\r\n', '\n' ) )
 
     def cmd_export( self, args ):
         force = args.getBooleanOption( '--force', False )
@@ -1360,6 +1366,7 @@ long_opt_info = {
     '--relocate': 0,            # relocate via URL-rewriting
     '--remove-ignored-items': 0,
     '--remove-unversioned-items': 0,
+    '--return-bytes': 0,
     '--revision': 1,            # revision X or X:Y range.  X or Y can be one of:
     '--revprop': 0,             # operate on a revision property (use with -r)
     '--search': 2,              # search for pattern
