@@ -3,13 +3,14 @@
 
 void IterT::init_type()
 {
-    behaviors().name( "IterT" );
+    behaviors().name( "pycxx_iter.IterT" );
     behaviors().doc( "IterT( ini_count )" );
     // you must have overwritten the virtual functions
     // Py::Object iter() and Py::Object iternext()
     behaviors().supportIter();    // set entries in the Type Table
     behaviors().supportRepr();
     add_varargs_method( "reversed", &IterT::reversed, "reversed()" );
+    behaviors().readyType();
 }
 
 class MyIterModule : public Py::ExtensionModule<MyIterModule>
@@ -22,7 +23,7 @@ public:
         add_varargs_method( "IterT", &MyIterModule::new_IterT, "IterT(from,last)" );
         initialize( "MyIterModule documentation" ); // register with Python
     }
-    
+
     virtual ~MyIterModule()
     {}
 
@@ -37,7 +38,13 @@ private:
     }
 };
 
-extern "C" PyObject *PyInit_pycxx_iter()
+#if defined( _WIN32 )
+#define EXPORT_SYMBOL __declspec( dllexport )
+#else
+#define EXPORT_SYMBOL
+#endif
+
+extern "C" EXPORT_SYMBOL PyObject *PyInit_pycxx_iter()
 {
     // the following constructor call registers our extension module
     // with the Python runtime system
