@@ -9,12 +9,10 @@ fi
 
 if [ "$1" != "" ]
 then
-    shift
     SVN_VER=$1
+    shift
 fi
 
-export MACOSX_DEPLOYMENT_TARGET=10.9
-
 case "$(uname)" in
 Linux)
     python${PY_VER} setup.py configure \
@@ -24,10 +22,12 @@ Linux)
     ;;
 
 Darwin)
+    export MACOSX_DEPLOYMENT_TARGET=11.0
+
     python${PY_VER} setup.py configure \
         --distro-dir=/usr/local/svn-${SVN_VER} \
         --distro-dir=/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.11.sdk/usr \
-        --arch=x86_64 \
+        --arch=x86_64 --arch=arm64 \
         --pycxx-dir=${BUILDER_TOP_DIR}/Import/pycxx-${PYCXX_VER} \
         --define=APR_IOVEC_DEFINED \
         --verbose \
