@@ -18,11 +18,12 @@ if sys.version_info[0] >= 3:
     module_name = 'pysvn.%s' % (module_name,)
 
 replacement = []
-if sys.platform == 'win32':
+if sys.platform == 'win32' and sys.version_info[0] >= 3 and sys.version_info[1] >= 8:
     replacement.append('''
     import os
     old_path = os.environ['PATH']
     os.environ['PATH'] = '%s;%s' % (os.path.dirname( __file__ ), old_path)
+    add_dll_handle = os.add_dll_directory( os.path.dirname( __file__ ) )
 ''')
 
 replacement.append( '    import %s\n' % (module_name,) )
@@ -30,7 +31,7 @@ replacement.append( '    import %s\n' % (module_name,) )
 if module_name != '_pysvn':
     replacement.append( '    _pysvn = %s\n' % (module_name,) )
 
-if sys.platform == 'win32':
+if sys.platform == 'win32' and sys.version_info[0] >= 3 and sys.version_info[1] >= 8:
     replacement.append('''
     os.environ['PATH'] = old_path
     del os
