@@ -719,10 +719,17 @@ public:
     // need a hash so that the enums can go into a map
     virtual long hash()
     {
-        static Py::String type_name( toTypeName( m_value ) );
+        static bool hash_extra_init = false;
+        static long hash_extra = 0;
+        if( !hash_extra_init )
+        {
+            Py::String type_name( toTypeName( m_value ) );
+            hash_extra = type_name.hashValue();
+            hash_extra_init = true;
+        }
 
         // use the m_value plus the hash of the type name
-        return long( m_value ) + type_name.hashValue();
+        return long( m_value ) + hash_extra;
     }
 
     static void init_type(void);
