@@ -36,6 +36,9 @@ Py::Object pysvn_client::cmd_add( const Py::Tuple &a_args, const Py::Dict &a_kws
     { false, name_depth },
     { false, name_add_parents },
 #endif
+#if defined( PYSVN_HAS_CLIENT_ADD5 )
+    { false, name_autoprops },
+#endif
 
     { false, NULL }
     };
@@ -56,6 +59,10 @@ Py::Object pysvn_client::cmd_add( const Py::Tuple &a_args, const Py::Dict &a_kws
 #else
     bool recurse = args.getBoolean( name_recurse, true );
 #endif
+#if defined( PYSVN_HAS_CLIENT_ADD5 )
+    bool autoprops = args.getBoolean( name_autoprops, true );
+#endif
+
     SvnPool pool( m_context );
     try
     {
@@ -70,7 +77,19 @@ Py::Object pysvn_client::cmd_add( const Py::Tuple &a_args, const Py::Dict &a_kws
 
             SvnPool pool( m_context );
 
-#if defined( PYSVN_HAS_CLIENT_ADD4 )
+#if defined( PYSVN_HAS_CLIENT_ADD5 )
+            svn_error_t * error = svn_client_add5
+                (
+                norm_path.c_str(),
+                depth,
+                force,
+                !ignore,
+                !autoprops,
+                add_parents,
+                m_context,
+                pool
+                );
+#elif defined( PYSVN_HAS_CLIENT_ADD4 )
             svn_error_t * error = svn_client_add4
                 (
                 norm_path.c_str(),
@@ -133,6 +152,10 @@ Py::Object pysvn_client::cmd_cat( const Py::Tuple &a_args, const Py::Dict &a_kws
     { false, name_revision },
 #if defined( PYSVN_HAS_CLIENT_CAT2 )
     { false, name_peg_revision },
+#endif
+#if defined( PYSVN_HAS_CLIENT_CAT3 )
+    { false, name_expand_keywords },
+    { false, name_get_props },
 #endif
     { false, NULL }
     };
@@ -156,6 +179,22 @@ Py::Object pysvn_client::cmd_cat( const Py::Tuple &a_args, const Py::Dict &a_kws
     svn_stringbuf_t * stringbuf = svn_stringbuf_create( empty_string, pool );
     svn_stream_t * stream = svn_stream_from_stringbuf( stringbuf, pool );
 
+#if defined( PYSVN_HAS_CLIENT_CAT3 )
+    bool get_props = args.getBoolean( name_get_props, false );
+    bool expand_keywords = args.getBoolean( name_expand_keywords, false );
+
+    apr_hash_t *props = NULL;
+    apr_hash_t **props_ptr;
+    if( get_props )
+    {
+        props_ptr = &props;
+    }
+    else
+    {
+        props_ptr = NULL;
+    }
+#endif
+
     try
     {
         std::string norm_path( svnNormalisedIfPath( path, pool ) );
@@ -163,7 +202,20 @@ Py::Object pysvn_client::cmd_cat( const Py::Tuple &a_args, const Py::Dict &a_kws
         checkThreadPermission();
 
         PythonAllowThreads permission( m_context );
-#if defined( PYSVN_HAS_CLIENT_CAT2 )
+#if defined( PYSVN_HAS_CLIENT_CAT3 )
+        svn_error_t *error = svn_client_cat3
+            (
+            props_ptr,
+            stream,
+            norm_path.c_str(),
+            &peg_revision,
+            &revision,
+            expand_keywords,
+            m_context,
+            pool,
+            pool
+            );
+#elif defined( PYSVN_HAS_CLIENT_CAT2 )
         svn_error_t *error = svn_client_cat2
             (
             stream,
@@ -198,7 +250,22 @@ Py::Object pysvn_client::cmd_cat( const Py::Tuple &a_args, const Py::Dict &a_kws
 
     // return the bytes as is to the application
     // we can assume nothing about them
-    return Py::Bytes( stringbuf->data, (int)stringbuf->len );
+    Py::Bytes contents( stringbuf->data, (int)stringbuf->len );
+#if defined( PYSVN_HAS_CLIENT_CAT3 )
+    if( get_props )
+    {
+        Py::Tuple result( 2 );
+
+        result[0] = contents;
+        result[1] = propsToObject( props, pool );
+
+        return result;
+    }
+    else
+#endif
+    {
+        return contents;
+    }
 }
 
 Py::Object pysvn_client::cmd_mkdir( const Py::Tuple &a_args, const Py::Dict &a_kws )
@@ -255,7 +322,12 @@ Py::Object pysvn_client::cmd_mkdir( const Py::Tuple &a_args, const Py::Dict &a_k
         throw Py::TypeError( type_error_message );
     }
 
+
+#if defined( PYSVN_HAS_CLIENT_MKDIR4 )
+    CommitInfoResult commit_info( pool );
+#else
     pysvn_commit_info_t *commit_info = NULL;
+#endif
 
     try
     {
@@ -268,7 +340,18 @@ Py::Object pysvn_client::cmd_mkdir( const Py::Tuple &a_args, const Py::Dict &a_k
             m_context.setLogMessage( message.c_str() );
         }
 
-#if defined( PYSVN_HAS_CLIENT_MKDIR3 )
+#if defined( PYSVN_HAS_CLIENT_MKDIR4 )
+        svn_error_t *error = svn_client_mkdir4
+            (
+            targets,
+            make_parents,
+            revprops,
+            commit_info.callback(),
+            commit_info.baton(),
+            m_context,
+            pool
+            );
+#elif defined( PYSVN_HAS_CLIENT_MKDIR3 )
         svn_error_t *error = svn_client_mkdir3
             (
             &commit_info,       // changed type
@@ -307,7 +390,11 @@ Py::Object pysvn_client::cmd_mkdir( const Py::Tuple &a_args, const Py::Dict &a_k
         throw_client_error( e );
     }
 
+#if defined( PYSVN_HAS_CLIENT_MKDIR4 )
+    return toObject( commit_info, m_wrapper_commit_info, m_commit_info_style );
+#else
     return toObject( commit_info, m_commit_info_style );
+#endif
 }
 
 Py::Object pysvn_client::cmd_remove( const Py::Tuple &a_args, const Py::Dict &a_kws )
@@ -345,14 +432,31 @@ Py::Object pysvn_client::cmd_remove( const Py::Tuple &a_args, const Py::Dict &a_
 
     apr_array_header_t *targets = targetsFromStringOrList( args.getArg( name_url_or_path ), pool );
 
+#if defined( PYSVN_HAS_CLIENT_MKDIR4 )
+    CommitInfoResult commit_info( pool );
+#else
     pysvn_commit_info_t *commit_info = NULL;
+#endif
+
     try
     {
         checkThreadPermission();
 
         PythonAllowThreads permission( m_context );
 
-#if defined( PYSVN_HAS_CLIENT_DELETE3 )
+#if defined( PYSVN_HAS_CLIENT_DELETE4 )
+        svn_error_t *error = svn_client_delete4
+            (
+            targets,
+            force,
+            keep_local,
+            revprops,
+            commit_info.callback(),
+            commit_info.baton(),
+            m_context,
+            pool
+            );
+#elif defined( PYSVN_HAS_CLIENT_DELETE3 )
         svn_error_t *error = svn_client_delete3
             (
             &commit_info,
@@ -394,7 +498,11 @@ Py::Object pysvn_client::cmd_remove( const Py::Tuple &a_args, const Py::Dict &a_
         throw_client_error( e );
     }
 
+#if defined( PYSVN_HAS_CLIENT_MKDIR4 )
+    return toObject( commit_info, m_wrapper_commit_info, m_commit_info_style );
+#else
     return toObject( commit_info, m_commit_info_style );
+#endif
 }
 
 Py::Object pysvn_client::cmd_revert( const Py::Tuple &a_args, const Py::Dict &a_kws )
@@ -406,6 +514,10 @@ Py::Object pysvn_client::cmd_revert( const Py::Tuple &a_args, const Py::Dict &a_
 #if defined( PYSVN_HAS_CLIENT_REVERT2 )
     { false, name_depth },
     { false, name_changelists },
+#endif
+#if defined( PYSVN_HAS_CLIENT_REVERT3 )
+    { false, name_clear_changelists },
+    { false, name_metadata_only },
 #endif
     { false, NULL }
     };
@@ -431,13 +543,28 @@ Py::Object pysvn_client::cmd_revert( const Py::Tuple &a_args, const Py::Dict &a_
         bool recurse = args.getBoolean( name_recurse, false );
 #endif
 
+#if defined( PYSVN_HAS_CLIENT_REVERT3 )
+    bool clear_changelists = args.getBoolean( name_clear_changelists, false );
+    bool metadata_only = args.getBoolean( name_metadata_only, false );
+#endif
         try
         {
             checkThreadPermission();
 
             PythonAllowThreads permission( m_context );
 
-#if defined( PYSVN_HAS_CLIENT_REVERT2 )
+#if defined( PYSVN_HAS_CLIENT_REVERT3 )
+            svn_error_t *error = svn_client_revert3
+                (
+                targets,
+                depth,
+                changelists,
+                clear_changelists,
+                metadata_only,
+                m_context,
+                pool
+                );
+#elif defined( PYSVN_HAS_CLIENT_REVERT2 )
             svn_error_t *error = svn_client_revert2
                 (
                 targets,
