@@ -235,6 +235,12 @@ Py::Object pysvn_client::cmd_checkout( const Py::Tuple &a_args, const Py::Dict &
     args.check();
 
     std::string url( args.getUtf8String( name_url ) );
+
+    if( !is_svn_url( url ) )
+    {
+        throw Py::AttributeError( "checkout url argument is not a valid SVN URL" );
+    }
+
     std::string path( args.getUtf8String( name_path ) );
 #if defined( PYSVN_HAS_CLIENT_CHECKOUT3 )
     bool allow_unver_obstructions = args.getBoolean( name_allow_unver_obstructions, false );
@@ -266,8 +272,6 @@ Py::Object pysvn_client::cmd_checkout( const Py::Tuple &a_args, const Py::Dict &
 
         PythonAllowThreads permission( m_context );
 
-        svn_revnum_t revnum = 0;
-
 #if defined( PYSVN_HAS_CLIENT_CHECKOUT3 )
         svn_error_t *error = svn_client_checkout3
             (
