@@ -214,6 +214,7 @@ Py::Object pysvn_client::cmd_diff( const Py::Tuple &a_args, const Py::Dict &a_kw
     { false, name_ignore_properties },
     { false, name_properties_only },
 #endif
+    { false, name_return_bytes },
     { false, NULL }
     };
     FunctionArguments args( "diff", args_desc, a_args, a_kws );
@@ -235,6 +236,8 @@ Py::Object pysvn_client::cmd_diff( const Py::Tuple &a_args, const Py::Dict &a_kw
     bool ignore_content_type = args.getBoolean( name_ignore_content_type, false );
 #endif
 
+    bool return_bytes = args.getBoolean( name_return_bytes, false );
+
     SvnPool pool( m_context );
 
 #if defined( PYSVN_HAS_CLIENT_DIFF3 )
@@ -434,8 +437,15 @@ Py::Object pysvn_client::cmd_diff( const Py::Tuple &a_args, const Py::Dict &a_kw
         throw_client_error( e );
     }
 
-    // cannot convert to Unicode as we have no idea of the encoding of the bytes
-    return Py::String( stringbuf->data, (int)stringbuf->len );
+    if( return_bytes )
+    {
+        return Py::Bytes( stringbuf->data, (int)stringbuf->len );
+    }
+    else
+    {
+        // assume that the data is UTF-8
+        return Py::String( stringbuf->data, (int)stringbuf->len, name_utf8 );
+    }
 }
 
 #if defined( PYSVN_HAS_CLIENT_DIFF_PEG )
@@ -472,6 +482,7 @@ Py::Object pysvn_client::cmd_diff_peg( const Py::Tuple &a_args, const Py::Dict &
     { false, name_ignore_properties },
     { false, name_properties_only },
 #endif
+    { false, name_return_bytes },
     { false, NULL }
     };
     FunctionArguments args( "diff_peg", args_desc, a_args, a_kws );
@@ -483,6 +494,8 @@ Py::Object pysvn_client::cmd_diff_peg( const Py::Tuple &a_args, const Py::Dict &
     svn_opt_revision_t revision_end = args.getRevision( name_revision_end, svn_opt_revision_working );
     svn_opt_revision_t peg_revision = args.getRevision( name_peg_revision, revision_end );
 
+    bool return_bytes = args.getBoolean( name_return_bytes, false );
+
     SvnPool pool( m_context );
 
 #if defined( PYSVN_HAS_CLIENT_DIFF_PEG4 )
@@ -710,8 +723,14 @@ Py::Object pysvn_client::cmd_diff_peg( const Py::Tuple &a_args, const Py::Dict &
         throw_client_error( e );
     }
 
-    // cannot convert to Unicode as we have no idea of the encoding of the bytes
-    return Py::String( stringbuf->data, (int)stringbuf->len );
+    if( return_bytes )
+    {
+        return Py::Bytes( stringbuf->data, (int)stringbuf->len );
+    }
+    else
+    {
+        return Py::String( stringbuf->data, (int)stringbuf->len, name_utf8 );
+    }
 }
 #endif
 
@@ -862,7 +881,6 @@ Py::Object pysvn_client::cmd_diff_summarize( const Py::Tuple &a_args, const Py::
         throw_client_error( e );
     }
 
-    // cannot convert to Unicode as we have no idea of the encoding of the bytes
     return diff_list;
 }
 
