@@ -22,7 +22,159 @@
 #include "svn_config.h"
 #include "pysvn_static_strings.hpp"
 
-static const char *g_utf_8 = "utf-8";
+#if defined( PYSVN_HAS_CLIENT_ANNOTATE5 )
+class AnnotatedLineInfo2
+{
+public:
+    AnnotatedLineInfo2
+        (
+        apr_int64_t line_no,
+        svn_revnum_t revision,
+        apr_hash_t *rev_props,
+        apr_hash_t *merged_rev_props,
+        svn_revnum_t merged_revision,
+        const char *merged_path,
+        const char *line,
+        svn_boolean_t local_change
+        )
+    : m_line_no( line_no )
+    , m_revision( revision )
+    , m_rev_props( rev_props ) //QQQ: need to copy?
+    , m_merged_revision( merged_revision)
+    , m_merged_rev_props( merged_rev_props ) //QQQ: need to copy?
+    , m_merged_path()
+    , m_line()
+    , m_local_change( local_change )
+    {
+    if( merged_path != NULL )
+        m_merged_path = merged_path;
+
+    if( line != NULL )
+        m_line = line;
+    }
+
+    ~AnnotatedLineInfo2()
+    {
+    }
+
+    AnnotatedLineInfo2( const AnnotatedLineInfo2 &other )
+    : m_line_no( other.m_line_no )
+    , m_revision( other.m_revision )
+    , m_rev_props( other.m_rev_props )
+    , m_merged_revision( other.m_merged_revision )
+    , m_merged_rev_props( other.m_merged_rev_props )
+    , m_merged_path( other.m_merged_path )
+    , m_line( other.m_line )
+    , m_local_change( other.m_local_change )
+    {
+    }
+
+    Py::Dict asDict( SvnPool &pool ) const
+    {
+        Py::Dict dict;
+        dict[name_line] = Py::String( m_line );
+        dict[name_number] = Py::Int( long( m_line_no ) );
+        dict[name_revision] = Py::asObject( new pysvn_revision( svn_opt_revision_number, 0, m_revision ) );
+        // rev_props
+        dict[name_local_change] = Py::Boolean( m_local_change );
+        if( SVN_IS_VALID_REVNUM( m_merged_revision ) )
+        {
+            dict[name_merged_revision] = Py::asObject( new pysvn_revision( svn_opt_revision_number, 0, m_merged_revision ) );
+            dict[name_merged_path] = path_string_or_none( m_merged_path, pool );
+            // merged_rev_props
+        }
+        else
+        {
+            dict[name_merged_revision] = Py::None();
+            dict[name_merged_path] = Py::None();
+            // merged_rev_props
+        }
+
+        return dict;
+    }
+
+    apr_int64_t     m_line_no;
+    svn_revnum_t    m_revision;
+    apr_hash_t     *m_rev_props;
+    svn_revnum_t    m_merged_revision;
+    apr_hash_t     *m_merged_rev_props;
+    std::string     m_merged_path;
+    std::string     m_line;
+    bool            m_local_change;
+};
+
+extern "C" svn_error_t *annotate3_receiver
+    (
+    void *baton,
+    svn_revnum_t start_revnum,
+    svn_revnum_t end_revnum,
+    apr_int64_t line_no,
+    svn_revnum_t revision,
+    apr_hash_t *rev_props,
+    svn_revnum_t merged_revision,
+    apr_hash_t *merged_rev_props,
+    const char *merged_path,
+    const char *line,
+    svn_boolean_t local_change,
+    apr_pool_t *pool
+    );
+class AnnotateBaton2
+{
+public:
+    AnnotateBaton2()
+    : m_all_entries()
+    {
+    }
+    ~AnnotateBaton2()
+    {
+    }
+
+    svn_client_blame_receiver3_t callback() { return &annotate3_receiver; }
+    void *baton() { return static_cast<void *>( this ); };
+    static AnnotateBaton2 *castBaton( void *baton_ ) { return static_cast<AnnotateBaton2 *>( baton_ ); }
+
+    std::list<AnnotatedLineInfo2> m_all_entries;
+};
+
+extern "C" svn_error_t *annotate3_receiver
+    (
+    void *baton,
+    svn_revnum_t start_revnum,
+    svn_revnum_t end_revnum,
+    apr_int64_t line_no,
+    svn_revnum_t revision,
+    apr_hash_t *rev_props,
+    svn_revnum_t merged_revision,
+    apr_hash_t *merged_rev_props,
+    const char *merged_path,
+    const char *line,
+    svn_boolean_t local_change,
+    apr_pool_t *pool
+    )
+{
+    // There are cases when the author has been passed as NULL
+    // protect against NULL passed for any of the strings
+    if( merged_path == NULL )
+        merged_path = "";
+    if( line == NULL )
+        line = "";
+    
+    AnnotateBaton2 *annotate_baton = AnnotateBaton2::castBaton( baton );
+
+    annotate_baton->m_all_entries.push_back(
+        AnnotatedLineInfo2(
+            line_no,
+            revision,
+            rev_props,
+            merged_rev_props,
+            merged_revision,
+            merged_path,
+            line,
+            local_change ) );
+
+    return NULL;
+}
+#endif
 
 #if defined( PYSVN_HAS_CLIENT_ANNOTATE4 )
 class AnnotatedLineInfo
@@ -97,7 +249,39 @@ public:
     std::string m_line;
 };
 
-static svn_error_t *annotate_receiver
+extern "C" svn_error_t *annotate2_receiver
+    (
+    void *baton,
+    apr_int64_t line_no,
+    svn_revnum_t revision,
+    const char *author,
+    const char *date,
+    svn_revnum_t merged_revision,
+    const char *merged_author,
+    const char *merged_date,
+    const char *merged_path,
+    const char *line,
+    apr_pool_t *pool
+    );
+class AnnotateBaton
+{
+public:
+    AnnotateBaton()
+    : m_all_entries()
+    {
+    }
+    ~AnnotateBaton()
+    {
+    }
+
+    svn_client_blame_receiver2_t callback() { return &annotate2_receiver; }
+    void *baton() { return static_cast<void *>( this ); };
+    static AnnotateBaton *castBaton( void *baton_ ) { return static_cast<AnnotateBaton *>( baton_ ); }
+
+    std::list<AnnotatedLineInfo> m_all_entries;
+};
+
+extern "C" svn_error_t *annotate2_receiver
     (
     void *baton,
     apr_int64_t line_no,
@@ -127,15 +311,17 @@ static svn_error_t *annotate_receiver
     if( line == NULL )
         line = "";
 
-    std::list<AnnotatedLineInfo> *entries = (std::list<AnnotatedLineInfo> *)baton;
-    entries->push_back( AnnotatedLineInfo(
-        line_no,
-        revision, author, date,
-        merged_revision, merged_author, merged_date, merged_path,
-        line ) );
+    AnnotateBaton *annotate_baton = AnnotateBaton::castBaton( baton );
+    annotate_baton->m_all_entries.push_back(
+                    AnnotatedLineInfo(
+                        line_no,
+                        revision, author, date,
+                        merged_revision, merged_author, merged_date, merged_path,
+                        line ) );
 
     return NULL;
 }
+
 #else
 class AnnotatedLineInfo
 {
@@ -176,7 +362,35 @@ public:
     std::string m_line;
 };
 
-static svn_error_t *annotate_receiver
+extern "C" svn_error_t *annotate1_receiver
+    (
+    void *baton,
+    apr_int64_t line_no,
+    svn_revnum_t revision,
+    const char *author,
+    const char *date,
+    const char *line,
+    apr_pool_t *pool
+    );
+
+class AnnotateBaton
+{
+public:
+    AnnotateBaton()
+    : m_all_entries()
+    {
+    }
+    ~AnnotateBaton()
+    {
+    }
+    svn_client_blame_receiver_t callback() { return &annotate1_receiver; }
+    void *baton() { return static_cast<void *>( this ); };
+    static AnnotateBaton *castBaton( void *baton_ ) { return static_cast<AnnotateBaton *>( baton_ ); }
+
+    std::list<AnnotatedLineInfo> m_all_entries;
+};
+
+extern "C" svn_error_t *annotate1_receiver
     (
     void *baton,
     apr_int64_t line_no,
@@ -196,13 +410,109 @@ static svn_error_t *annotate_receiver
     if( line == NULL )
         line = "";
 
-    std::list<AnnotatedLineInfo> *entries = (std::list<AnnotatedLineInfo> *)baton;
-    entries->push_back( AnnotatedLineInfo( line_no, revision, author, date, line ) );
+    AnnotateBaton *annotate_baton = AnnotateBaton::castBaton( baton );
+    annotate_baton->m_all_entries.push_back( AnnotatedLineInfo( line_no, revision, author, date, line ) );
 
     return NULL;
 }
 #endif
 
+#if defined( PYSVN_HAS_CLIENT_ANNOTATE5 )
+Py::Object pysvn_client::cmd_annotate2( const Py::Tuple &a_args, const Py::Dict &a_kws )
+{
+    static argument_description args_desc[] =
+    {
+    { true,  name_url_or_path },
+    { false, name_revision_start },
+    { false, name_revision_end },
+    { false, name_peg_revision },
+    { false, name_ignore_space },
+    { false, name_ignore_eol_style },
+    { false, name_ignore_mime_type },
+    { false, name_include_merged_revisions },
+    { false, NULL }
+    };
+    FunctionArguments args( "annotate", args_desc, a_args, a_kws );
+    args.check();
+
+    std::string path( args.getUtf8String( name_url_or_path, empty_string ) );
+    svn_opt_revision_t revision_start = args.getRevision( name_revision_start, svn_opt_revision_number );
+    svn_opt_revision_t revision_end = args.getRevision( name_revision_end, svn_opt_revision_head );
+    svn_opt_revision_t peg_revision = args.getRevision( name_peg_revision, revision_end );
+
+    svn_diff_file_ignore_space_t ignore_space = svn_diff_file_ignore_space_none;
+    if( args.hasArg( name_ignore_space ) )
+    {
+        Py::ExtensionObject< pysvn_enum_value<svn_diff_file_ignore_space_t> > py_ignore_space( args.getArg( name_ignore_space ) );
+        ignore_space = svn_diff_file_ignore_space_t( py_ignore_space.extensionObject()->m_value );
+    }
+
+    svn_boolean_t ignore_eol_style = args.getBoolean( name_ignore_eol_style, false );
+    svn_boolean_t ignore_mime_type = args.getBoolean( name_ignore_mime_type, false );
+    svn_boolean_t include_merged_revisions = args.getBoolean( name_include_merged_revisions, false );
+    SvnPool pool( m_context );
+
+    svn_diff_file_options_t *diff_options = svn_diff_file_options_create( pool );
+    diff_options->ignore_space = ignore_space;
+    diff_options->ignore_eol_style = ignore_eol_style;
+
+    bool is_url = is_svn_url( path );
+    revisionKindCompatibleCheck( is_url, peg_revision, name_peg_revision, name_url_or_path );
+    revisionKindCompatibleCheck( is_url, revision_start, name_revision_start, name_url_or_path );
+    revisionKindCompatibleCheck( is_url, revision_end, name_revision_end, name_url_or_path );
+
+    AnnotateBaton2 annotate_baton;
+
+    try
+    {
+        std::string norm_path( svnNormalisedIfPath( path, pool ) );
+
+        checkThreadPermission();
+
+        PythonAllowThreads permission( m_context );
+
+        svn_error_t *error = svn_client_blame5
+            (
+            norm_path.c_str(),
+            &peg_revision,
+            &revision_start,
+            &revision_end,
+            diff_options,
+            ignore_mime_type,
+            include_merged_revisions,
+            annotate_baton.callback(),
+            annotate_baton.baton(),
+            m_context,
+            pool
+            );
+
+        permission.allowThisThread();
+        if( error != NULL )
+        {
+            throw SvnException( error );
+        }
+    }
+    catch( SvnException &e )
+    {
+        // use callback error over ClientException
+        m_context.checkForError( m_module.client_error );
+
+        throw_client_error( e );
+    }
+
+    // convert the entries into python objects
+    Py::List all_lines;
+    for( std::list<AnnotatedLineInfo2>::const_iterator entry_it = annotate_baton.m_all_entries.begin();
+        entry_it != annotate_baton.m_all_entries.end();
+        ++entry_it )
+    {
+        all_lines.append( entry_it->asDict( pool ) );
+    }
+
+    return all_lines;
+}
+#endif
+
 Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &a_kws )
 {
     static argument_description args_desc[] =
@@ -261,7 +571,7 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
     revisionKindCompatibleCheck( is_url, revision_start, name_revision_start, name_url_or_path );
     revisionKindCompatibleCheck( is_url, revision_end, name_revision_end, name_url_or_path );
 
-    std::list<AnnotatedLineInfo> all_entries;
+    AnnotateBaton annotate_baton;
 
     try
     {
@@ -272,7 +582,7 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
         PythonAllowThreads permission( m_context );
 
 #if defined( PYSVN_HAS_CLIENT_ANNOTATE4 )
-        svn_error_t *error = svn_client_blame4
+        svn_error_t *error = svn_client_blame4 // ignore deprecation warning - support old Client().annotate()
             (
             norm_path.c_str(),
             &peg_revision,
@@ -281,8 +591,8 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
             diff_options,
             ignore_mime_type,
             include_merged_revisions,
-            annotate_receiver,
-            &all_entries,
+            annotate_baton.callback(),
+            annotate_baton.baton(),
             m_context,
             pool
             );
@@ -296,8 +606,8 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
             diff_options,
             ignore_mime_type,
             annotate_receiver,
-            &all_entries,
-            m_context,
+            annotate_baton.callback(),
+            annotate_baton.baton(),
             pool
             );
 #elif defined( PYSVN_HAS_CLIENT_ANNOTATE2 )
@@ -307,8 +617,8 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
             &peg_revision,
             &revision_start,
             &revision_end,
-            annotate_receiver,
-            &all_entries,
+            annotate_baton.callback(),
+            annotate_baton.baton(),
             m_context,
             pool
             );
@@ -318,8 +628,8 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
             norm_path.c_str(),
             &revision_start,
             &revision_end,
-            annotate_receiver,
-            &all_entries,
+            annotate_baton.callback(),
+            annotate_baton.baton(),
             m_context,
             pool
             );
@@ -340,8 +650,8 @@ Py::Object pysvn_client::cmd_annotate( const Py::Tuple &a_args, const Py::Dict &
 
     // convert the entries into python objects
     Py::List entries_list;
-    std::list<AnnotatedLineInfo>::const_iterator entry_it = all_entries.begin();
-    while( entry_it != all_entries.end() )
+    std::list<AnnotatedLineInfo>::const_iterator entry_it = annotate_baton.m_all_entries.begin();
+    while( entry_it != annotate_baton.m_all_entries.end() )
     {
         const AnnotatedLineInfo &entry = *entry_it;
         ++entry_it;
@@ -386,7 +696,17 @@ Py::Object pysvn_client::cmd_info( const Py::Tuple &a_args, const Py::Dict &a_kw
 #if defined( PYSVN_HAS_WC_ADM_PROBE_OPEN3 )
         const char *c_norm_path = svn_dirent_internal_style( path.c_str(), pool );
         std::string norm_path( c_norm_path );
-        svn_error_t *error = svn_wc_adm_probe_open3( &adm_access, NULL, norm_path.c_str(), false, 0, NULL, NULL, pool );
+        svn_error_t *error = svn_wc_adm_probe_open3 // ignore deprecation warning - support old Client().info()
+                    (
+                    &adm_access,
+                    NULL,
+                    norm_path.c_str(),
+                    false,
+                    0,
+                    NULL,
+                    NULL,
+                    pool
+                    );
 #else
         std::string norm_path( svnNormalisedPath( path, pool ) );
         svn_error_t *error = svn_wc_adm_probe_open( &adm_access, NULL, norm_path.c_str(), false, false, pool );
@@ -397,7 +717,14 @@ Py::Object pysvn_client::cmd_info( const Py::Tuple &a_args, const Py::Dict &a_kw
             throw SvnException( error );
 
         permission.allowOtherThreads();
-        error = svn_wc_entry( &entry, norm_path.c_str(), adm_access, false, pool );
+        error = svn_wc_entry // ignore deprecation warning - support old Client().info()
+                    (
+                    &entry,
+                    norm_path.c_str(),
+                    adm_access,
+                    false,
+                    pool
+                    );
         permission.allowThisThread();
         if( error != NULL )
             throw SvnException( error );
@@ -418,13 +745,83 @@ Py::Object pysvn_client::cmd_info( const Py::Tuple &a_args, const Py::Dict &a_kw
     return toObject( *entry, pool, m_wrapper_entry );
 }
 
-#if defined( PYSVN_HAS_CLIENT_INFO )
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+extern "C" svn_error_t *info_receiver_c2( void *baton_, const char *abspath_or_url, const svn_client_info2_t *info, apr_pool_t *pool );
+
 class InfoReceiveBaton
 {
 public:
     InfoReceiveBaton
         (
         PythonAllowThreads *permission,
+        SvnPool &pool,
+        Py::List &info_list,
+        const DictWrapper &wrapper_info,
+        const DictWrapper &wrapper_lock,
+        const DictWrapper &wrapper_wc_info
+        )
+    : m_permission( permission )
+    , m_pool( pool )
+    , m_info_list( info_list )
+    , m_wrapper_info( wrapper_info )
+    , m_wrapper_lock( wrapper_lock )
+    , m_wrapper_wc_info( wrapper_wc_info )
+    {}
+    ~InfoReceiveBaton()
+    {}
+
+    svn_client_info_receiver2_t callback() { return &info_receiver_c2; }
+    void *baton() { return static_cast<void *>( this ); }
+    static InfoReceiveBaton *castBaton( void *baton_ ) { return static_cast<InfoReceiveBaton *>( baton_ ); }
+
+    PythonAllowThreads  *m_permission;
+    SvnPool             &m_pool;
+    Py::List            &m_info_list;
+    const DictWrapper   &m_wrapper_info;
+    const DictWrapper   &m_wrapper_lock;
+    const DictWrapper   &m_wrapper_wc_info;
+};
+
+extern "C" svn_error_t *info_receiver_c2( void *baton_, const char *abspath_or_url, const svn_client_info2_t *info, apr_pool_t * )
+{
+    InfoReceiveBaton *baton = InfoReceiveBaton::castBaton( baton_ );
+
+    PythonDisallowThreads callback_permission( baton->m_permission );
+
+    if( abspath_or_url != NULL )
+    {
+        std::string std_path( abspath_or_url );
+        if( std_path.empty() )
+        {
+            std_path = ".";
+        }
+        Py::String py_path( utf8_string_or_none( std_path ) );
+
+        Py::Tuple py_pair( 2 );
+        py_pair[0] = py_path;
+        py_pair[1] = toObject(
+                    *info,
+                    baton->m_pool,
+                    baton->m_wrapper_info,
+                    baton->m_wrapper_lock,
+                    baton->m_wrapper_wc_info );
+
+        baton->m_info_list.append( py_pair );
+    }
+
+    return SVN_NO_ERROR;
+}
+
+#elif defined( PYSVN_HAS_CLIENT_INFO )
+extern "C" svn_error_t *info_receiver_c( void *baton_, const char *path, const svn_info_t *info, apr_pool_t *pool );
+
+class InfoReceiveBaton
+{
+public:
+    InfoReceiveBaton
+        (
+        PythonAllowThreads *permission,
+        SvnPool &,
         Py::List &info_list,
         const DictWrapper &wrapper_info,
         const DictWrapper &wrapper_lock,
@@ -439,6 +836,10 @@ public:
     ~InfoReceiveBaton()
     {}
 
+    svn_info_receiver_t callback() { return &info_receiver_c; }
+    void *baton() { return static_cast<void *>( this ); }
+    static InfoReceiveBaton *castBaton( void *baton_ ) { return static_cast<InfoReceiveBaton *>( baton_ ); }
+
     PythonAllowThreads  *m_permission;
     Py::List            &m_info_list;
     const DictWrapper   &m_wrapper_info;
@@ -446,11 +847,9 @@ public:
     const DictWrapper   &m_wrapper_wc_info;
 };
 
-extern "C"
+extern "C" svn_error_t *info_receiver_c( void *baton_, const char *path, const svn_info_t *info, apr_pool_t *pool )
 {
-static svn_error_t *info_receiver_c( void *baton_, const char *path, const svn_info_t *info, apr_pool_t *pool )
-{
-    InfoReceiveBaton *baton = reinterpret_cast<InfoReceiveBaton *>( baton_ );
+    InfoReceiveBaton *baton = InfoReceiveBaton::castBaton( baton_ );
 
     PythonDisallowThreads callback_permission( baton->m_permission );
 
@@ -461,7 +860,7 @@ static svn_error_t *info_receiver_c( void *baton_, const char *path, const svn_i
         {
             std_path = ".";
         }
-        Py::String py_path( std_path );
+        Py::String py_path( utf8_string_or_none( std_path ) );
 
         Py::Tuple py_pair( 2 );
         py_pair[0] = py_path;
@@ -476,8 +875,9 @@ static svn_error_t *info_receiver_c( void *baton_, const char *path, const svn_i
 
     return SVN_NO_ERROR;
 }
-}
+#endif
 
+#if defined( PYSVN_HAS_CLIENT_INFO )
 Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_kws )
 {
     static argument_description args_desc[] =
@@ -489,6 +889,13 @@ Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_k
 #if defined( PYSVN_HAS_CLIENT_INFO2 )
     { false, name_depth },
     { false, name_changelists },
+#endif
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+    { false, name_fetch_excluded },
+    { false, name_fetch_actual_only },
+#endif
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+    { false, name_include_externals },
 #endif
     { false, NULL }
     };
@@ -519,6 +926,14 @@ Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_k
     bool recurse = args.getBoolean( name_recurse, true );
 #endif
 
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+    bool fetch_excluded = args.getBoolean( name_fetch_excluded, false );
+    bool fetch_actual_only = args.getBoolean( name_fetch_actual_only, true );
+#endif
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+    bool include_externals = args.getBoolean( name_include_externals, false );
+#endif
+
     bool is_url = is_svn_url( path );
     revisionKindCompatibleCheck( is_url, peg_revision, name_peg_revision, name_url_or_path );
     revisionKindCompatibleCheck( is_url, revision, name_revision, name_url_or_path );
@@ -533,17 +948,69 @@ Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_k
 
         PythonAllowThreads permission( m_context );
 
-        InfoReceiveBaton info_baton( &permission, info_list, m_wrapper_info, m_wrapper_lock, m_wrapper_wc_info );
+        InfoReceiveBaton info_baton( &permission, pool, info_list, m_wrapper_info, m_wrapper_lock, m_wrapper_wc_info );
 
-#if defined( PYSVN_HAS_CLIENT_INFO2 )
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+        svn_error_t *error = SVN_NO_ERROR;
+        const char *abspath_or_url = NULL;
+        if( !svn_path_is_url( norm_path.c_str() )
+        &&  !svn_dirent_is_absolute( norm_path.c_str() ) )
+        {
+            error = svn_dirent_get_absolute( &abspath_or_url, norm_path.c_str(), pool );
+        }
+        else
+        {
+            abspath_or_url = norm_path.c_str();
+        }
+#endif
+
+
+#if defined( PYSVN_HAS_CLIENT_INFO4 )
+        if( error == SVN_NO_ERROR )
+        {
+            error = svn_client_info4
+                    (
+                    abspath_or_url,
+                    &peg_revision,
+                    &revision,
+                    depth,
+                    fetch_excluded,
+                    fetch_actual_only,
+                    include_externals,
+                    changelists,
+                    info_baton.callback(),
+                    info_baton.baton(),
+                    m_context,
+                    pool
+                    );
+        }
+#elif defined( PYSVN_HAS_CLIENT_INFO3 )
+        if( error == SVN_NO_ERROR )
+        {
+            error = svn_client_info3
+                    (
+                    abspath_or_url,
+                    &peg_revision,
+                    &revision,
+                    depth,
+                    fetch_excluded,
+                    fetch_actual_only,
+                    changelists,
+                    info_baton.callback(),
+                    info_baton.baton(),
+                    m_context,
+                    pool
+                    );
+        }
+#elif defined( PYSVN_HAS_CLIENT_INFO2 )
         svn_error_t *error = 
             svn_client_info2
                 (
                 norm_path.c_str(),
                 &peg_revision,
                 &revision,
-                info_receiver_c,
-                reinterpret_cast<void *>( &info_baton ),
+                info_baton.callback(),
+                info_baton.baton(),
                 depth,
                 changelists,
                 m_context,
@@ -556,8 +1023,8 @@ Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_k
                 norm_path.c_str(),
                 &peg_revision,
                 &revision,
-                info_receiver_c,
-                reinterpret_cast<void *>( &info_baton ),
+                info_baton.callback(),
+                info_baton.baton(),
                 recurse,
                 m_context,
                 pool
@@ -580,6 +1047,13 @@ Py::Object pysvn_client::cmd_info2( const Py::Tuple &a_args, const Py::Dict &a_k
 #endif
 
 #if defined( PYSVN_HAS_CLIENT_LOG4 )
+extern "C" svn_error_t *log4Receiver
+    (
+    void *baton_,
+    svn_log_entry_t *log_entry,
+    apr_pool_t *pool
+    );
+
 class Log4Baton
 {
 public:
@@ -595,6 +1069,10 @@ public:
     ~Log4Baton()
         {}
 
+    svn_log_entry_receiver_t callback() { return &log4Receiver; }
+    void *baton() { return static_cast< void * >( this ); }
+    static Log4Baton *castBaton( void *baton_ ) { return static_cast<Log4Baton *>( baton_ ); }
+
     PythonAllowThreads  *m_permission;
     SvnPool             &m_pool;
     apr_time_t          m_now;
@@ -604,14 +1082,14 @@ public:
     bool                m_has_children;
 };
 
-static svn_error_t *log4Receiver
+extern "C" svn_error_t *log4Receiver
     (
     void *baton_,
     svn_log_entry_t *log_entry,
     apr_pool_t *pool
     )
 {
-    Log4Baton *baton = reinterpret_cast<Log4Baton *>( baton_ );
+    Log4Baton *baton = Log4Baton::castBaton( baton_ );
 
     if( log_entry->revision == 0 )
     {
@@ -638,7 +1116,7 @@ static svn_error_t *log4Receiver
         if( revprops_dict.hasKey( "svn:date" ) )
         {
             Py::String date( revprops_dict[ "svn:date" ] );
-            Py::Object int_date = toObject( convertStringToTime( date.as_std_string( g_utf_8 ), baton->m_now, baton->m_pool ) );
+            Py::Object int_date = toObject( convertStringToTime( date.as_std_string( name_utf8 ), baton->m_now, baton->m_pool ) );
             revprops_dict[ "svn:date" ] = int_date;
             entry_dict[ name_date ] = int_date;
         }
@@ -825,8 +1303,8 @@ Py::Object pysvn_client::cmd_log( const Py::Tuple &a_args, const Py::Dict &a_kws
             strict_node_history,
             include_merged_revisions,
             revprops,
-            log4Receiver,
-            reinterpret_cast<void *>( &baton ),
+            baton.callback(),
+            baton.baton(),
             m_context,
             pool
             );
@@ -842,8 +1320,8 @@ Py::Object pysvn_client::cmd_log( const Py::Tuple &a_args, const Py::Dict &a_kws
             strict_node_history,
             include_merged_revisions,
             revprops,
-            log4Receiver,
-            reinterpret_cast<void *>( &baton ),
+            baton.callback(),
+            baton.baton(),
             m_context,
             pool
             );
@@ -1182,14 +1660,247 @@ Py::Object pysvn_client::cmd_log( const Py::Tuple &a_args, const Py::Dict &a_kws
 }
 #endif
 
-struct StatusEntriesBaton
+#if defined( PYSVN_HAS_CLIENT_STATUS5 )
+static svn_error_t *status5EntriesFunc
+    (
+    void *baton,
+    const char *path,
+    const svn_client_status_t *status,
+    apr_pool_t *scratch_pool
+    );
+
+class Status2EntriesBaton
 {
-    apr_pool_t* pool;
-    apr_hash_t* hash;
+public:
+    Status2EntriesBaton( SvnPool &pool )
+    : m_pool( pool )
+    , m_hash( apr_hash_make( pool ) )
+    {}
+
+    ~Status2EntriesBaton()
+    {}
+
+    svn_client_status_func_t callback() { return &status5EntriesFunc; }
+
+    void *baton() { return static_cast< void * >( this ); }
+    static Status2EntriesBaton *castBaton( void *baton_ ) { return static_cast<Status2EntriesBaton *>( baton_ ); }
+
+    SvnPool     &m_pool;
+    apr_hash_t  *m_hash;
+};
+
+static svn_error_t *status5EntriesFunc
+    (
+    void *baton,
+    const char *path,
+    const svn_client_status_t *status,
+    apr_pool_t *scratch_pool
+    )
+{
+    Status2EntriesBaton *seb = Status2EntriesBaton::castBaton( baton );
+
+    path = apr_pstrdup( seb->m_pool, path );
+    svn_client_status_t *stat = svn_client_status_dup( status, seb->m_pool );
+    apr_hash_set( seb->m_hash, path, APR_HASH_KEY_STRING, stat );
+    return SVN_NO_ERROR;
+}
+
+Py::Object pysvn_client::cmd_status2( const Py::Tuple &a_args, const Py::Dict &a_kws )
+{
+    static argument_description args_desc[] =
+    {
+    { true,  name_path },
+    { false, name_recurse },
+    { false, name_get_all },
+    { false, name_update },
+    { false, name_ignore },
+    { false, name_ignore_externals },
+    { false, name_depth },
+    { false, name_changelists },
+    { false, name_depth_as_sticky },
+#if defined( PYSVN_HAS_CLIENT_STATUS6 )
+    { false, name_check_out_of_date },
+    { false, name_check_working_copy },
+#endif
+    { false, NULL }
+    };
+    FunctionArguments args( "status2", args_desc, a_args, a_kws );
+    args.check();
+
+    Py::String path( args.getUtf8String( name_path ) );
+
+    SvnPool pool( m_context );
+
+    apr_array_header_t *changelists = NULL;
+
+    if( args.hasArg( name_changelists ) )
+    {
+        changelists = arrayOfStringsFromListOfStrings( args.getArg( name_changelists ), pool );
+    }
+
+    svn_depth_t depth = args.getDepth( name_depth, name_recurse, svn_depth_infinity, svn_depth_infinity, svn_depth_immediates );
+    bool get_all = args.getBoolean( name_get_all, true );
+    bool update = args.getBoolean( name_update, false );
+    bool ignore = args.getBoolean( name_ignore, false );
+    bool ignore_externals = args.getBoolean( name_ignore_externals, false );
+    bool depth_as_sticky = args.getBoolean( name_depth_as_sticky, true );
+
+#if defined( PYSVN_HAS_CLIENT_STATUS6 )
+    bool  check_out_of_date = args.getBoolean( name_check_out_of_date, update );
+    bool  check_working_copy = args.getBoolean( name_check_working_copy, true );
+#endif
+
+    Status2EntriesBaton baton( pool );
+
+    Py::List entries_list;
+    try
+    {
+        std::string norm_path( svnNormalisedIfPath( path, pool ) );
+
+        checkThreadPermission();
+
+        PythonAllowThreads permission( m_context );
+
+        svn_revnum_t revnum;
+        svn_opt_revision_t rev = { svn_opt_revision_head, {0} };
+
+
+        const char *abspath_or_url = NULL;
+        svn_error_t *error = svn_dirent_get_absolute( &abspath_or_url, norm_path.c_str(), pool );
+
+#if defined( PYSVN_HAS_CLIENT_STATUS6 )
+        if( error == NULL )
+        {
+            error = svn_client_status6
+                (
+                &revnum,            // revnum
+                m_context,
+                abspath_or_url,     // path
+                &rev,
+                depth,
+                get_all,
+                check_out_of_date,
+                check_working_copy,
+                !ignore,
+                ignore_externals,
+                depth_as_sticky,
+                changelists,
+                baton.callback(),
+                baton.baton(),
+                pool
+                );
+        }
+#else
+        if( error == NULL )
+        {
+            error = svn_client_status5
+                (
+                &revnum,            // revnum
+                m_context,
+                abspath_or_url,     // path
+                &rev,
+                depth,
+                get_all,
+                update,
+                !ignore,
+                ignore_externals,
+                depth_as_sticky,
+                changelists,
+                baton.callback(),
+                baton.baton(),
+                pool
+                );
+        }
+#endif
+        permission.allowThisThread();
+        if( error != NULL )
+            throw SvnException( error );
+    }
+    catch( SvnException &e )
+    {
+        // use callback error over ClientException
+        m_context.checkForError( m_module.client_error );
+
+        throw_client_error( e );
+    }
+
+    // Loop over array, returning each name/status-structure
+
+    for( apr_hash_index_t *hi = apr_hash_first( pool, baton.m_hash );
+            hi != NULL;
+                hi = apr_hash_next( hi ) )
+    {
+        const void *key;
+        void *val;
+        apr_hash_this( hi, &key, NULL, &val );
+
+        svn_client_status_t *status = static_cast<svn_client_status_t *>( val );
+
+        entries_list.append( toObject(
+                Py::String( osNormalisedPath( (const char *)key, pool ), "UTF-8" ),
+                *status,
+                pool,
+                m_wrapper_status2,
+                m_wrapper_lock ) );
+    }
+
+    entries_list.sort();
+
+    return entries_list;
+}
+#endif
+
+#if defined( PYSVN_HAS_CLIENT_STATUS4 )
+static svn_error_t *status4EntriesFunc
+    (
+    void *baton,
+    const char *path,
+    svn_wc_status2_t *status,
+    apr_pool_t *pool
+    );
+#elif defined( PYSVN_HAS_CLIENT_STATUS2 )
+static void status2EntriesFunc
+    (
+    void *baton,
+    const char *path,
+    svn_wc_status2_t *status
+    );
+#else
+static void status1EntriesFunc
+    (
+    void *baton,
+    const char *path,
+    svn_wc_status_t *status
+    );
+#endif
+
+class StatusEntriesBaton
+{
+public:
+    StatusEntriesBaton( SvnPool &pool )
+    : m_pool( pool )
+    , m_hash( apr_hash_make( pool ) )
+    {}
+
+    ~StatusEntriesBaton()
+    {}
+
+#if defined( PYSVN_HAS_CLIENT_STATUS4 )
+    svn_wc_status_func3_t callback() { return &status4EntriesFunc; }
+#elif defined( PYSVN_HAS_CLIENT_STATUS2 )
+    svn_wc_status_func2_t callback() { return &status2EntriesFunc; }
+#else
+    svn_wc_status_func_t callback() { return &status1EntriesFunc; }
+#endif
+    void *baton() { return static_cast< void * >( this ); }
+    static StatusEntriesBaton *castBaton( void *baton_ ) { return static_cast<StatusEntriesBaton *>( baton_ ); }
+
+    SvnPool     &m_pool;
+    apr_hash_t  *m_hash;
 };
 
 #if defined( PYSVN_HAS_CLIENT_STATUS4 )
-static svn_error_t *StatusEntriesFunc
+static svn_error_t *status4EntriesFunc
     (
     void *baton,
     const char *path,
@@ -1197,43 +1908,40 @@ static svn_error_t *StatusEntriesFunc
     apr_pool_t *pool
     )
 {
-    svn_wc_status2_t *stat;
-    StatusEntriesBaton *seb = reinterpret_cast<StatusEntriesBaton *>( baton );
+    StatusEntriesBaton *seb = StatusEntriesBaton::castBaton( baton );
 
-    path = apr_pstrdup( seb->pool, path );
-    stat = svn_wc_dup_status2( status, seb->pool );
-    apr_hash_set( seb->hash, path, APR_HASH_KEY_STRING, stat );
+    path = apr_pstrdup( seb->m_pool, path );
+    svn_wc_status2_t *stat = svn_wc_dup_status2( status, seb->m_pool ); // ignore deprecation needed for Client().status()
+    apr_hash_set( seb->m_hash, path, APR_HASH_KEY_STRING, stat );
     return SVN_NO_ERROR;
 }
 #elif defined( PYSVN_HAS_CLIENT_STATUS2 )
-static void StatusEntriesFunc
+static void status2EntriesFunc
     (
     void *baton,
     const char *path,
     svn_wc_status2_t *status
     )
 {
-    svn_wc_status2_t *stat;
-    StatusEntriesBaton *seb = reinterpret_cast<StatusEntriesBaton *>( baton );
+    StatusEntriesBaton *seb = StatusEntriesBaton::castBaton( baton );
 
-    path = apr_pstrdup( seb->pool, path );
-    stat = svn_wc_dup_status2( status, seb->pool );
-    apr_hash_set( seb->hash, path, APR_HASH_KEY_STRING, stat );
+    path = apr_pstrdup( seb->m_pool, path );
+    svn_wc_status2_t *stat = svn_wc_dup_status2( status, seb->m_pool );
+    apr_hash_set( seb->m_hash, path, APR_HASH_KEY_STRING, stat );
 }
 #else
-static void StatusEntriesFunc
+static void status1EntriesFunc
     (
     void *baton,
     const char *path,
     svn_wc_status_t *status
     )
 {
-    svn_wc_status_t *stat;
-    StatusEntriesBaton *seb = reinterpret_cast<StatusEntriesBaton *>( baton );
+    StatusEntriesBaton *seb = StatusEntriesBaton::castBaton( baton );
 
-    path = apr_pstrdup( seb->pool, path );
-    stat = svn_wc_dup_status( status, seb->pool );
-    apr_hash_set( seb->hash, path, APR_HASH_KEY_STRING, stat );
+    path = apr_pstrdup( seb->m_pool, path );
+    svn_wc_status2_t *stat = svn_wc_dup_status( status, seb->m_pool );
+    apr_hash_set( seb->m_hash, path, APR_HASH_KEY_STRING, stat );
 }
 #endif
 
@@ -1281,7 +1989,7 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
     bool ignore_externals = args.getBoolean( name_ignore_externals, false );
 #endif
 
-    apr_hash_t *status_hash = NULL;
+    StatusEntriesBaton baton( pool );
 
     Py::List entries_list;
     try
@@ -1295,20 +2003,14 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
         svn_revnum_t revnum;
         svn_opt_revision_t rev = { svn_opt_revision_head, {0} };
 
-        StatusEntriesBaton baton;
-
-        status_hash = apr_hash_make( pool );
-        baton.hash = status_hash;
-        baton.pool = pool;
-
 #if defined( PYSVN_HAS_CLIENT_STATUS4 )
-        svn_error_t *error = svn_client_status4
+        svn_error_t *error = svn_client_status4 // ignore deprecation warning - support Client().status()
             (
             &revnum,            // revnum
             norm_path.c_str(),  // path
             &rev,
-            StatusEntriesFunc,  // status func
-            &baton,             // status baton
+            baton.callback(),
+            baton.baton(),
             depth,
             get_all,
             update,
@@ -1324,8 +2026,8 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
             &revnum,            // revnum
             norm_path.c_str(),  // path
             &rev,
-            StatusEntriesFunc,  // status func
-            &baton,             // status baton
+            baton.callback(),
+            baton.baton(),
             depth,
             get_all,
             update,
@@ -1341,8 +2043,8 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
             &revnum,            // revnum
             norm_path.c_str(),  // path
             &rev,
-            StatusEntriesFunc,  // status func
-            &baton,             // status baton
+            baton.callback(),
+            baton.baton(),
             recurse,
             get_all,
             update,
@@ -1357,8 +2059,8 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
             &revnum,            // revnum
             norm_path.c_str(),  // path
             &rev,
-            StatusEntriesFunc,  // status func
-            &baton,             // status baton
+            baton.callback(),
+            baton.baton(),
             recurse,
             get_all,
             update,
@@ -1381,7 +2083,7 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
 
     // Loop over array, returning each name/status-structure
 
-    for( apr_hash_index_t *hi = apr_hash_first( pool, status_hash );
+    for( apr_hash_index_t *hi = apr_hash_first( pool, baton.m_hash );
             hi != NULL;
                 hi = apr_hash_next( hi ) )
     {
@@ -1389,8 +2091,7 @@ Py::Object pysvn_client::cmd_status( const Py::Tuple &a_args, const Py::Dict &a_
         void *val;
         apr_hash_this( hi, &key, NULL, &val );
 
-        pysvn_wc_status_t *status = (pysvn_wc_status_t *)val;
-
+        pysvn_wc_status_t *status = static_cast<pysvn_wc_status_t *>( val );
         entries_list.append( toObject(
                 Py::String( osNormalisedPath( (const char *)key, pool ), "UTF-8" ),
                 *status,
