@@ -27,6 +27,9 @@ Py::Object pysvn_client::cmd_relocate( const Py::Tuple &a_args, const Py::Dict &
     { true,  name_to_url },
     { true,  name_path },
     { false, name_recurse },
+#if defined( PYSVN_HAS_CLIENT_RELOCATE2 )
+    { false, name_ignore_externals },
+#endif
     { false, NULL }
     };
     FunctionArguments args( "relocate", args_desc, a_args, a_kws );
@@ -36,7 +39,11 @@ Py::Object pysvn_client::cmd_relocate( const Py::Tuple &a_args, const Py::Dict &
     std::string to_url( args.getUtf8String( name_to_url ) );
     std::string path( args.getUtf8String( name_path ) );
 
+#if defined( PYSVN_HAS_CLIENT_RELOCATE2 )
+    bool ignore_externals = args.getBoolean( name_ignore_externals, true );
+#else
     bool recurse = args.getBoolean( name_recurse, true );
+#endif
 
     SvnPool pool( m_context );
 
@@ -50,6 +57,18 @@ Py::Object pysvn_client::cmd_relocate( const Py::Tuple &a_args, const Py::Dict &
 
         PythonAllowThreads permission( m_context );
 
+#if defined( PYSVN_HAS_CLIENT_RELOCATE2 )
+        svn_error_t *error = svn_client_relocate2
+            (
+            norm_path.c_str(),
+            norm_from_url.c_str(),
+            norm_to_url.c_str(),
+            ignore_externals,
+            m_context,
+            pool
+            );
+
+#else
         svn_error_t *error = svn_client_relocate
             (
             norm_path.c_str(),
@@ -59,6 +78,7 @@ Py::Object pysvn_client::cmd_relocate( const Py::Tuple &a_args, const Py::Dict &
             m_context,
             pool
             );
+#endif
         permission.allowThisThread();
 
         if( error != NULL )
@@ -89,6 +109,9 @@ Py::Object pysvn_client::cmd_switch( const Py::Tuple &a_args, const Py::Dict &a_
     { false, name_depth_is_sticky },
     { false, name_ignore_externals },
     { false, name_allow_unver_obstructions },
+#endif
+#if defined( PYSVN_HAS_CLIENT_SWITCH3 )
+    { false, name_ignore_ancestry },
 #endif
     { false, NULL }
     };
@@ -108,6 +131,11 @@ Py::Object pysvn_client::cmd_switch( const Py::Tuple &a_args, const Py::Dict &a_
 #else
     bool recurse = args.getBoolean( name_recurse, true );
 #endif
+
+#if defined( PYSVN_HAS_CLIENT_SWITCH3 )
+    bool ignore_ancestry = args.getBoolean( name_ignore_ancestry, false );
+#endif
+
     SvnPool pool( m_context );
 
     svn_revnum_t revnum = 0;
@@ -120,7 +148,24 @@ Py::Object pysvn_client::cmd_switch( const Py::Tuple &a_args, const Py::Dict &a_
 
         PythonAllowThreads permission( m_context );
 
-#if defined( PYSVN_HAS_CLIENT_SWITCH2 )
+#if defined( PYSVN_HAS_CLIENT_SWITCH3 )
+        svn_error_t *error = svn_client_switch3
+            (
+            &revnum,
+            norm_path.c_str(),
+            norm_url.c_str(),
+            &peg_revision,
+            &revision,
+            depth,
+            depth_is_sticky,
+            ignore_externals,
+            allow_unver_obstructions,
+            ignore_ancestry,
+            m_context,
+            pool
+            );
+
+#elif defined( PYSVN_HAS_CLIENT_SWITCH2 )
         svn_error_t *error = svn_client_switch2
             (
             &revnum,
