@@ -14,9 +14,7 @@
 #
 import sys
 import os
-import distutils
-import distutils.sysconfig
-import distutils.util
+import sysconfig
 
 import xml.dom.minidom
 import xml.sax
@@ -1042,12 +1040,12 @@ class MacOsxCompilerGCC(CompilerGCC):
 
         self._find_paths_pycxx_dir = [
                         '../Import/pycxx-%d.%d.%d' % min_pycxx_version,
-                        distutils.sysconfig.get_python_inc() # typical Linux
+                        sysconfig.get_path('include') # typical Linux
                         ]
 
         self._find_paths_pycxx_src = [
                         '%(PYCXX)s/Src',
-                        '/usr/share/python%s/CXX' % (distutils.sysconfig.get_python_version(),) # typical Linux
+                        '/usr/share/python%s/CXX' % (sysconfig.get_python_version(),) # typical Linux
                         ]
 
         if self.options.hasOption( '--distro-dir' ):
@@ -1129,7 +1127,7 @@ class MacOsxCompilerGCC(CompilerGCC):
         self._addVar( 'PYTHON_VERSION',     '%d.%d' % (sys.version_info[0], sys.version_info[1]) )
         self._addVar( 'PYTHON_DIR',         sys.exec_prefix )
         self._addVar( 'PYTHON_FRAMEWORK',   os.path.join( python_prefix, 'Python' ) )
-        self._addVar( 'PYTHON_INC',         distutils.sysconfig.get_python_inc() )
+        self._addVar( 'PYTHON_INC',         sysconfig.get_path('include') )
 
         py_cflags_list = [
                     '-g',
@@ -1185,12 +1183,12 @@ class UnixCompilerGCC(CompilerGCC):
         CompilerGCC.__init__( self, setup )
 
         self._find_paths_pycxx_dir = [
-                        distutils.sysconfig.get_python_inc(), # typical Linux
+                        sysconfig.get_path('include'), # typical Linux
                         '/usr/include'
                         ]
         self._find_paths_pycxx_src = [
                         '%(PYCXX)s/Src',
-                        '/usr/share/python%s/CXX' % distutils.sysconfig.get_python_version(), # typical Linux
+                        '/usr/share/python%s/CXX' % sysconfig.get_python_version(), # typical Linux
                         '/usr/src/CXX'
                         ]
 
@@ -1254,8 +1252,8 @@ class UnixCompilerGCC(CompilerGCC):
         self._addVar( 'PYSVN_MODULE_BASENAME', self.pysvn_module_name )
 
         self._addVar( 'PYTHON_VERSION', '%d.%d' % (sys.version_info[0], sys.version_info[1]) )
-        self._addVar( 'PYTHON_INC',     distutils.sysconfig.get_python_inc( False ) )
-        self._addVar( 'PYTHON_ARCH_SPECIFIC_INC',     distutils.sysconfig.get_python_inc( True ) )
+        self._addVar( 'PYTHON_INC',     sysconfig.get_path('include') )
+        self._addVar( 'PYTHON_ARCH_SPECIFIC_INC',     sysconfig.get_path('platinclude') )
 
         py_ldflags_list = []
 
